#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent

# import interfaces
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_default_theme.component.illustration.interfaces import IIllustrationWithZoomSettings, IIllustrationRenderer, \
    ILLUSTRATION_AFTER_BODY, ILLUSTRATION_BEFORE_BODY
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.features.renderer.skin import BaseContentRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from zope.interface import implementer
from zope.location import locate, Location
from zope.schema.fieldproperty import FieldProperty

from pyams_default_theme import _


#
# Illustration renderer with zoom settings
#

ILLUSTRATION_ZOOM_RENDERER_SETTINGS_KEY = 'pyams_content.illustration.renderer:zoom'


@implementer(IIllustrationWithZoomSettings)
class IllustrationZoomSettings(Persistent, Location):
    """Illustration zoom renderer settings"""

    zoom_on_click = FieldProperty(IIllustrationWithZoomSettings['zoom_on_click'])


@adapter_config(context=IIllustration, provides=IIllustrationWithZoomSettings)
def illustration_with_zoom_settings_factory(context):
    """Illustration zoom renderer settings factory"""
    return get_annotation_adapter(context, ILLUSTRATION_ZOOM_RENDERER_SETTINGS_KEY, IllustrationZoomSettings)


#
# Illustration renderers
#

@implementer(IIllustrationRenderer)
class BaseIllustrationRenderer(BaseContentRenderer):
    """Base illustration renderer"""

    context_attrs = ('author', )
    i18n_context_attrs = ('title', 'alt_title', 'description', 'data')

    position = None


@adapter_config(name='centered-before-body', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class DefaultIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("Centered illustration before text")
    weight = 1

    position = ILLUSTRATION_BEFORE_BODY


@adapter_config(name='left+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-left.pt', layer=IPyAMSLayer)
class LeftIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("Small illustration on the left")
    weight = 2

    position = ILLUSTRATION_BEFORE_BODY
    settings_interface = IIllustrationWithZoomSettings


@adapter_config(name='right+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-right.pt', layer=IPyAMSLayer)
class RightIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("Small illustration on the right")
    weight = 3

    position = ILLUSTRATION_BEFORE_BODY
    settings_interface = IIllustrationWithZoomSettings


@adapter_config(name='default', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class DefaultIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("Centered illustration after text")
    weight = 4

    position = ILLUSTRATION_AFTER_BODY
